from django_actionable_messages.adaptive_card.outlook.mixins import ElementMixin
from django_actionable_messages.adaptive_card.utils import HorizontalAlignment, SpacingStyle


class ActionSet(ElementMixin):
    def __init__(self, item_id: str = None, spacing: SpacingStyle = None, separator: bool = None,
                 horizontal_alignment: HorizontalAlignment = None, actions=None, **kwargs):
        self._data = {
            "type": "ActionSet"
        }
        super().__init__(**kwargs)
        if item_id is not None:
            self.set_id(item_id)
        if spacing is not None:
            self.set_spacing(spacing)
        if separator is not None:
            self.set_separator(separator)
        if horizontal_alignment is not None:
            self. set_horizontal_alignment(horizontal_alignment)
        if actions:
            self.add_actions(actions)

    def set_id(self, item_id: str):
        self._data["id"] = item_id

    def set_spacing(self, spacing: SpacingStyle):
        self._data["spacing"] = spacing

    def set_separator(self, value=True):
        self._data["separator"] = value

    def set_horizontal_alignment(self, horizontal_alignment: HorizontalAlignment):
        self._data["horizontalAlignment"] = horizontal_alignment

    def add_actions(self, actions):
        self._data.setdefault("actions", [])
        if isinstance(actions, (list, set, tuple)):
            self._data["actions"].extend(self._get_items_list(actions))
        else:
            self._data["actions"].append(actions.as_data())



from typing import Union

from django_actionable_messages.adaptive_card.mixins import ActionMixin, BaseElementMixin
from django_actionable_messages.adaptive_card.utils import FallbackOption, ActionStyle, ActionMode, AssociatedInputs
from django_actionable_messages.exceptions import CardException
from django_actionable_messages.mixins import CardElement


class OpenUrl(ActionMixin):
    action_type = "Action.OpenUrl"

    def __init__(self, url: str, **kwargs):
        super().__init__(**kwargs)
        self.set_url(url)

    def set_url(self, url: str):
        self._data["url"] = url


class Submit(ActionMixin):
    action_type = "Action.Submit"

    def __init__(self, data: Union[str, dict] = None, **kwargs):
        super().__init__(**kwargs)
        if data is not None:
            self.set_data(data)

    def set_data(self, data: Union[str, dict]):
        self._data["data"] = data


class ShowCard(ActionMixin):
    action_type = "Action.ShowCard"

    def __init__(self, card=None, **kwargs):
        super().__init__(**kwargs)
        if card is not None:
            self.set_card(card)

    def set_card(self, card):
        self._data["card"] = card.payload


class TargetElement(CardElement):
    def __init__(self, element_id: str, is_visible: bool = None, **kwargs):
        self._data = {
            "elementId": element_id
        }
        super().__init__(**kwargs)
        if is_visible is not None:
            self.set_is_visible(is_visible)

    def set_element_id(self, element_id: str):
        self._data["elementId"] = element_id

    def set_is_visible(self, value=True):
        self._data["isVisible"] = value


class ToggleVisibility(ActionMixin):
    action_type = "Action.ToggleVisibility"

    def __init__(self, target_elements: list = None, **kwargs):
        super().__init__(**kwargs)
        if target_elements:
            self.set_target_elements(target_elements)

    def set_target_elements(self, elements: list):
        self._data["targetElements"] = []
        for element in elements:
            if isinstance(element, TargetElement):
                self._data["targetElements"].append(element.as_data())
            else:
                self._data["targetElements"].append(element)


class Execute(CardElement):
    action_type = "Action.Execute"

    def __init__(self, verb: str = None, data: Union[str, object] = None, associated_inputs: AssociatedInputs = None,
                 title: str = None, icon_url: str = None, style: ActionStyle = None, fallback=None,
                 tooltip: str = None, is_enabled: bool = None, mode: ActionMode = None,
                 requires: dict = None, **kwargs):
        self._data = {
            "type": self.action_type
        }
        super().__init__(**kwargs)
        if verb is not None:
            self.set_verb(verb)
        if data is not None:
            self.set_data(data)
        if associated_inputs is not None:
            self.set_associated_inputs(associated_inputs)
        if title is not None:
            self.set_title(title)
        if icon_url is not None:
            self.set_icon_url(icon_url)
        if style is not None:
            self.set_style(style)
        if fallback is not None:
            self.set_fallback(fallback)
        if tooltip is not None:
            self.set_tooltip(tooltip)
        if is_enabled is not None:
            self.set_is_enabled(is_enabled)
        if mode is not None:
            self.set_mode(mode)
        if requires is not None:
            self.set_requires(requires)

    def set_verb(self, verb: str):
        self._data["verb"] = verb

    def set_data(self, data: Union[str, object]):
        self._data["data"] = data

    def set_associated_inputs(self, associated_inputs: AssociatedInputs):
        self._data["associatedInputs"] = associated_inputs

    def set_title(self, title: str):
        self._data["title"] = title

    def set_icon_url(self, icon_url: str):
        self._data["iconUrl"] = icon_url

    def set_style(self, style: ActionStyle):
        self._data["style"] = style

    def set_fallback(self, fallback):
        if isinstance(fallback, FallbackOption):
            self._data["fallback"] = fallback
        elif isinstance(fallback, BaseElementMixin):
            self._data["fallback"] = fallback.as_data()
        else:
            raise CardException("Invalid fallback type")

    def set_tooltip(self, tooltip: str):
        self._data["tooltip"] = tooltip

    def set_is_enabled(self, is_enabled: bool):
        self._data["isEnabled"] = is_enabled

    def set_mode(self, mode: ActionMode):
        self._data["mode"] = mode

    def set_requires(self, requires: dict):
        self._data["requires"] = requires



import uuid
from typing import List, Union

from django_actionable_messages.exceptions import CardException
from django_actionable_messages.mixins import MESSAGE_CARD, Card


class MessageCard(Card):
    card_type = MESSAGE_CARD

    def __init__(self, title=None, text=None, originator: str = None, summary=None, theme_color: str = None,
                 correlation_id: str = None, auto_correlation_id=True, expected_actors: List[str] = None,
                 hide_original_body: bool = None, sections: list = None, actions: list = None, **kwargs):
        self._payload = {
            "@type": "MessageCard",
            "@context": "https://schema.org/extensions"
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)
        if text is not None:
            self.set_text(text)
        if originator is not None:
            self.set_originator(originator)
        if summary is not None:
            self.set_summary(summary)
        if theme_color is not None:
            self.set_theme_color(theme_color)
        if correlation_id is not None or auto_correlation_id:
            self._payload["correlationId"] = correlation_id or str(uuid.uuid4())
        if expected_actors:
            self.set_expected_actors(expected_actors)
        if hide_original_body is not None:
            self.set_hide_original_body(hide_original_body)
        if sections:
            self.add_sections(sections)
        if actions:
            self.add_actions(actions)

    def set_title(self, title):
        self._payload["title"] = title

    def set_text(self, text):
        self._payload["text"] = text

    def set_originator(self, originator: str):
        self._payload["originator"] = originator

    def set_summary(self, summary):
        self._payload["summary"] = summary

    def set_theme_color(self, theme_color: str):
        self._payload["themeColor"] = theme_color

    def set_correlation_id(self, correlation_id: str):
        self._payload["correlationId"] = correlation_id

    def set_expected_actors(self, expected_actors: List[str]):
        self._payload["expectedActors"] = expected_actors

    def add_expected_actors(self, expected_actors: Union[str, List[str]]):
        self._payload.setdefault("expectedActors", [])
        if isinstance(expected_actors, (list, set, tuple)):
            self._payload["expectedActors"].extend(list(expected_actors))
        elif isinstance(expected_actors, str):
            self._payload["expectedActors"].append(expected_actors)
        else:
            raise CardException("Invalid expected_actors type")

    def set_hide_original_body(self, hide_original_body=True):
        self._payload["hideOriginalBody"] = hide_original_body

    def add_sections(self, sections):
        self._payload.setdefault("sections", [])
        if isinstance(sections, (list, set, tuple)):
            self._payload["sections"].extend(self._get_items_list(sections))
        else:
            self._payload["sections"].append(sections.as_data())

    def add_actions(self, actions):
        self._payload.setdefault("potentialAction", [])
        if isinstance(actions, (list, set, tuple)):
            self._payload["potentialAction"].extend(self._get_items_list(actions))
        else:
            self._payload["potentialAction"].append(actions.as_data())



from django_actionable_messages.adaptive_card.outlook.mixins import ElementMixin, DisplayFormMixin
from django_actionable_messages.exceptions import CardException

METHODS = ("GET", "POST")


class ActionHttp(ElementMixin):
    def __init__(self, method: str, url: str, title=None, headers: list = None, body=None, **kwargs):
        if method not in METHODS:
            raise CardException("Invalid method. Available methods are: {}".format(METHODS))
        if method == "POST" and not body:
            raise CardException("If method is POST body must be provided")
        self._data = {
            "type": "Action.Http",
            "method": method,
            "url": url
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)
        if headers:
            self.add_headers(headers)
        if body is not None:
            self.set_body(body)

    def set_title(self, title):
        self._data["title"] = title

    def add_headers(self, headers):
        self._data.setdefault("headers", [])
        if isinstance(headers, (list, set, tuple)):
            self._data["headers"].extend(self._get_items_list(headers))
        else:
            self._data["headers"].append(headers.as_data())

    def set_body(self, body):
        self._data["body"] = body


class InvokeAddInCommand(ElementMixin):
    def __init__(self, add_in_id: str, desktop_command_id: str, initialization_context, title=None, **kwargs):
        self._data = {
            "type": "Action.InvokeAddInCommand",
            "addInId": add_in_id,
            "desktopCommandId": desktop_command_id,
            "initializationContext": initialization_context
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)

    def set_add_in_id(self, add_in_id: str):
        self._data["addInId"] = add_in_id

    def set_desktop_command_id(self, cmd_id: str):
        self._data["desktopCommandId"] = cmd_id

    def set_initialization_context(self, context):
        self._data["initializationContext"] = context

    def set_title(self, title):
        self._data["title"] = title


class DisplayMessageForm(DisplayFormMixin):
    base_type = "Action.DisplayMessageForm"


class DisplayAppointmentForm(DisplayFormMixin):
    base_type = "Action.DisplayAppointmentForm"


class ToggleVisibility(ElementMixin):
    def __init__(self, target_elements, title=None, **kwargs):
        self._data = {
            "type": "Action.ToggleVisibility"
        }
        self.add_target_elements(target_elements)
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)

    def set_title(self, title):
        self._data["title"] = title

    def add_target_elements(self, elements):
        self._data.setdefault("targetElements", [])
        if isinstance(elements, (list, set, tuple)):
            self._data["targetElements"].extend(self._get_items_list(elements))
        else:
            self._data["targetElements"].append(elements.as_data())



from django_actionable_messages.adaptive_card.utils import (
    FallbackOption, BlockElementHeight, SpacingStyle, ActionStyle, ActionMode
)
from django_actionable_messages.exceptions import CardException
from django_actionable_messages.mixins import CardElement


class BaseElementMixin(CardElement):
    def __init__(self, fallback=None, separator: bool = None, spacing: SpacingStyle = None, item_id: str = None,
                 is_visible: bool = None, requires: dict = None, **kwargs):
        super().__init__(**kwargs)
        if fallback is not None:
            self.set_fallback(fallback)
        if separator is not None:
            self.set_separator(separator)
        if spacing is not None:
            self.set_spacing(spacing)
        if item_id is not None:
            self.set_id(item_id)
        if is_visible is not None:
            self.set_is_visible(is_visible)
        if requires is not None:
            self.set_requires(requires)

    def set_fallback(self, fallback):
        if isinstance(fallback, FallbackOption):
            self._data["fallback"] = fallback
        elif isinstance(fallback, BaseElementMixin):
            self._data["fallback"] = fallback.as_data()
        else:
            raise CardException("Invalid fallback type")

    def set_separator(self, value=True):
        self._data["separator"] = value

    def set_spacing(self, spacing: SpacingStyle):
        self._data["spacing"] = spacing

    def set_id(self, item_id: str):
        self._data["id"] = item_id

    def set_is_visible(self, visible=True):
        self._data["isVisible"] = visible

    def set_requires(self, requires: dict):
        self._data["requires"] = requires


class ElementMixin(BaseElementMixin):
    def __init__(self, height: BlockElementHeight = None, **kwargs):
        super().__init__(**kwargs)
        if height is not None:
            self.set_height(height)

    def set_height(self, height: BlockElementHeight):
        self._data["height"] = height


class InputMixin(ElementMixin):
    def __init__(self, label=None, **kwargs):
        super().__init__(**kwargs)
        if label is not None:
            self.set_label(label)

    def set_label(self, label):
        if isinstance(label, str):
            self._data["label"] = label
        else:
            self._data["label"] = label.as_data()


class DateTimeMixin(InputMixin):
    base_type = None

    def __init__(self, max_value: str = None, min_value: str = None, placeholder=None, value: str = None, **kwargs):
        self._data = {
            "type": self.base_type
        }
        super().__init__(**kwargs)
        if max_value is not None:
            self.set_max(max_value)
        if min_value is not None:
            self.set_min(min_value)
        if placeholder is not None:
            self.set_placeholder(placeholder)
        if value is not None:
            self.set_value(value)

    def set_max(self, value: str):
        self._data["max"] = value

    def set_min(self, value: str):
        self._data["min"] = value

    def set_placeholder(self, text):
        self._data["placeholder"] = text

    def set_value(self, value: str):
        self._data["value"] = value


class ActionMixin(CardElement):
    action_type = ""

    def __init__(self, title=None, icon_url: str = None, style: ActionStyle = None, fallback=None, tooltip: str = None,
                 is_enabled: bool = None, mode: ActionMode = None, requires: dict = None, **kwargs):
        self._data = {
            "type": self.action_type
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)
        if icon_url is not None:
            self.set_icon_url(icon_url)
        if style is not None:
            self.set_style(style)
        if fallback is not None:
            self.set_fallback(fallback)
        if tooltip is not None:
            self.set_tooltip(tooltip)
        if is_enabled is not None:
            self.set_is_enabled(is_enabled)
        if mode is not None:
            self.set_mode(mode)
        if requires is not None:
            self.set_requires(requires)

    def set_title(self, title):
        self._data["title"] = title

    def set_icon_url(self, url: str):
        self._data["iconUrl"] = url

    def set_style(self, style: ActionStyle):
        self._data["style"] = style

    def set_fallback(self, fallback):
        if isinstance(fallback, FallbackOption):
            self._data["fallback"] = fallback
        elif isinstance(fallback, ActionMixin):
            self._data["fallback"] = fallback.as_data()
        else:
            raise CardException("Invalid fallback type")

    def set_tooltip(self, tooltip: str):
        self._data["tooltip"] = tooltip

    def set_is_enabled(self, is_enabled: bool):
        self._data["isEnabled"] = is_enabled

    def set_mode(self, mode: ActionMode):
        self._data["mode"] = mode

    def set_requires(self, requires: dict):
        self._data["requires"] = requires



from typing import List

from django_actionable_messages.adaptive_card.utils import HorizontalAlignment, VerticalAlignment, FillMode
from django_actionable_messages.mixins import CardElement


class BackgroundImage(CardElement):
    def __init__(self, url: str, fill_mode: FillMode = None, horizontal_alignment: HorizontalAlignment = None,
                 vertical_alignment: VerticalAlignment = None, **kwargs):
        self._data = {
            "url": url
        }
        super().__init__(**kwargs)
        if fill_mode is not None:
            self.set_fill_mode(fill_mode)
        if horizontal_alignment is not None:
            self.set_horizontal_alignment(horizontal_alignment)
        if vertical_alignment is not None:
            self.set_vertical_alignment(vertical_alignment)

    def set_url(self, url: str):
        self._data["url"] = url

    def set_fill_mode(self, fill_mode: FillMode):
        self._data["fillMode"] = fill_mode

    def set_horizontal_alignment(self, alignment: HorizontalAlignment):
        self._data["horizontalAlignment"] = alignment

    def set_vertical_alignment(self, alignment: VerticalAlignment):
        self._data["verticalAlignment"] = alignment


class Refresh(CardElement):
    def __init__(self, action=None, user_ids: List[str] = None, **kwargs):
        self._data = {}
        super().__init__(**kwargs)
        if action is not None:
            self.set_action(action)
        if user_ids is not None:
            self.set_user_ids(user_ids)

    def set_action(self, action):
        self._data["action"] = action.as_data()

    def set_user_ids(self, user_ids: List[str]):
        self._data["userIds"] = user_ids


class TokenExchangeResource(CardElement):
    def __init__(self, token_id: str, uri: str, provider_id: str, **kwargs):
        self._data = {
            "id": token_id,
            "uri": uri,
            "providerId": provider_id
        }
        super().__init__(**kwargs)

    def set_id(self, token_id: str):
        self._data["id"] = token_id

    def set_uri(self, uri: str):
        self._data["uri"] = uri

    def set_provider_id(self, provider_id: str):
        self._data["providerId"] = provider_id


class AuthCardButton(CardElement):
    def __init__(self, btn_type: str, value: str, title: str = None, image: str = None, **kwargs):
        self._data = {
            "type": btn_type,
            "value": value
        }
        super().__init__(**kwargs)
        if title is not None:
            self.set_title(title)
        if image is not None:
            self.set_image(image)

    def set_title(self, title: str):
        self._data["title"] = title

    def set_image(self, image: str):
        self._data["image"] = image


class Authentication(CardElement):
    def __init__(self, text: str = None, connection_name: str = None,
                 token_exchange_resource: TokenExchangeResource = None,
                 buttons: List[AuthCardButton] = None, **kwargs):
        self._data = {}
        super().__init__(**kwargs)
        if text is not None:
            self.set_text(text)
        if connection_name is not None:
            self.set_connection_name(connection_name)
        if token_exchange_resource is not None:
            self.set_token_exchange_resource(token_exchange_resource)
        if buttons is not None:
            self.set_buttons(buttons)

    def set_text(self, text: str):
        self._data["text"] = text

    def set_connection_name(self, connection_name: str):
        self._data["connectionName"] = connection_name

    def set_token_exchange_resource(self, token_exchange_resource: TokenExchangeResource):
        self._data["tokenExchangeResource"] = token_exchange_resource.as_data()

    def set_buttons(self, buttons: List[AuthCardButton]):
        self._data["buttons"] = self._get_items_list(buttons)



from django_actionable_messages.mixins import HERO_CARD, THUMBNAIL_CARD
from django_actionable_messages.msteams_cards.mixins import CardMixin


class HeroCard(CardMixin):
    card_type = HERO_CARD
    content_type = "application/vnd.microsoft.card.hero"


class ThumbnailCard(CardMixin):
    card_type = THUMBNAIL_CARD
    content_type = "application/vnd.microsoft.card.thumbnail"



import copy
import json

from django.template.loader import render_to_string

from django_actionable_messages.settings import card_settings

MESSAGE_CARD = 1
ADAPTIVE_CARD = 2
HERO_CARD = 3
THUMBNAIL_CARD = 4


class BaseMixin:
    def __init__(self, *args, **kwargs):
        self.language_code = kwargs.pop("lang_code", card_settings.LANGUAGE_CODE)
        super().__init__()

    def get_language_code(self):
        return self.language_code

    def _get_items_list(self, items):
        return list([item.as_data() for item in items])


class CardElement(BaseMixin):
    _data = None

    def as_data(self):
        return self._data


class Card(BaseMixin):
    json_encoder = card_settings.JSON_ENCODER

    _payload = None
    card_type = None
    script_types = {
        MESSAGE_CARD: "application/ld+json",
        ADAPTIVE_CARD: "application/adaptivecard+json"
    }
    signed_card_types = {
        MESSAGE_CARD: "SignedMessageCard",
        ADAPTIVE_CARD: "SignedAdaptiveCard"
    }

    @property
    def payload(self):
        return self.get_payload()

    @property
    def json_payload(self):
        return self.get_payload(fmt="json")

    @property
    def html_payload(self):
        return self.get_payload(fmt="html")

    @property
    def signed_html_payload(self):
        return self.get_payload(fmt="signed_html")

    def get_payload(self, fmt=None):
        payload = copy.deepcopy(self._payload)
        if fmt == "json":
            payload = self._get_json_payload(payload)
        elif fmt == "html":
            payload = self._get_html_payload(payload)
        elif fmt == "signed_html":
            payload = self._get_signed_html_payload()
        return payload

    def get_signed_payload(self):
        raise NotImplementedError

    def get_json_dump_kwargs(self):
        return {}

    def _get_json_payload(self, payload):
        kwargs = {
            "cls": self.json_encoder,
            "lang_code": self.get_language_code()
        }
        kwargs.update(**self.get_json_dump_kwargs())
        return json.dumps(payload, **kwargs)

    def _get_html_payload(self, payload):
        context = {
            "type": self.script_types[self.card_type],
            "payload": self._get_json_payload(payload)
        }
        return render_to_string("django_actionable_messages/email.html", context=context)

    def _get_signed_html_payload(self):
        context = {
            "type": self.signed_card_types[self.card_type],
            "payload": self.get_signed_payload()
        }
        return render_to_string("django_actionable_messages/signed_email.html", context=context)
